@@ -20,8 +20,8 @@
 """
 
 version_string = """\
-Nuitka V2.0.6
-Copyright (C) 2023 Kay Hayen."""
+Nuitka V2.1
+Copyright (C) 2024 Kay Hayen."""
 
 
 def getNuitkaVersion():
