@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 # We are not avoiding these in generated code at all
 # pylint: disable=I0021,too-many-lines
 # pylint: disable=I0021,line-too-long
@@ -36,6 +22,7 @@ from nuitka.Options import shallMakeModule
 from nuitka.PythonVersions import python_version
 from nuitka.specs.BuiltinParameterSpecs import extractBuiltinArgs
 from nuitka.specs.HardImportSpecs import (
+    builtins_open_since_3_spec,
     ctypes_cdll_before_38_spec,
     ctypes_cdll_since_38_spec,
     importlib_metadata_backport_distribution_spec,
@@ -47,12 +34,16 @@ from nuitka.specs.HardImportSpecs import (
     importlib_metadata_entry_points_since_310_spec,
     importlib_metadata_metadata_spec,
     importlib_metadata_version_spec,
+    importlib_resources_backport_files_spec,
+    importlib_resources_backport_read_binary_spec,
+    importlib_resources_backport_read_text_spec,
     importlib_resources_files_spec,
     importlib_resources_read_binary_spec,
     importlib_resources_read_text_spec,
     os_listdir_spec,
     os_path_abspath_spec,
     os_path_basename_spec,
+    os_path_dirname_spec,
     os_path_exists_spec,
     os_path_isabs_spec,
     os_path_isdir_spec,
@@ -64,11 +55,13 @@ from nuitka.specs.HardImportSpecs import (
     pkg_resources_resource_stream_spec,
     pkg_resources_resource_string_spec,
     pkgutil_get_data_spec,
+    sys_exit_spec,
 )
 
 from .ChildrenHavingMixins import (
     ChildHavingDistMixin,
     ChildHavingDistributionNameMixin,
+    ChildHavingExitCodeOptionalMixin,
     ChildHavingPackageMixin,
     ChildHavingParamsTupleMixin,
     ChildHavingPathMixin,
@@ -76,6 +69,7 @@ from .ChildrenHavingMixins import (
     ChildHavingPMixin,
     ChildHavingRequirementsTupleMixin,
     ChildHavingSMixin,
+    ChildrenHavingFileModeOptionalBufferingOptionalEncodingOptionalErrorsOptionalNewlineOptionalClosefdOptionalOpenerOptionalMixin,
     ChildrenHavingGroupNameOptionalMixin,
     ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalMixin,
     ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalWinmodeOptionalMixin,
@@ -95,6 +89,126 @@ from .ImportHardNodes import ExpressionImportModuleNameHardExistsSpecificBase
 hard_import_node_classes = {}
 
 
+class ExpressionBuiltinsOpenRef(ExpressionImportModuleNameHardExistsSpecificBase):
+    """Function reference builtins.open"""
+
+    kind = "EXPRESSION_BUILTINS_OPEN_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name="builtins",
+            import_name="open",
+            module_guaranteed=True,
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .BuiltinOpenNodes import makeExpressionBuiltinsOpenCall
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=makeExpressionBuiltinsOpenCall,
+            builtin_spec=builtins_open_since_3_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'builtins.open' recognized.",
+        )
+
+
+hard_import_node_classes[ExpressionBuiltinsOpenRef] = builtins_open_since_3_spec
+
+
+class ExpressionBuiltinsOpenCallBase(
+    ChildrenHavingFileModeOptionalBufferingOptionalEncodingOptionalErrorsOptionalNewlineOptionalClosefdOptionalOpenerOptionalMixin,
+    ExpressionBase,
+):
+    """Base class for BuiltinsOpenCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = (
+        "file",
+        "mode|optional",
+        "buffering|optional",
+        "encoding|optional",
+        "errors|optional",
+        "newline|optional",
+        "closefd|optional",
+        "opener|optional",
+    )
+
+    __slots__ = ("attempted",)
+
+    spec = builtins_open_since_3_spec
+
+    def __init__(
+        self,
+        file,
+        mode,
+        buffering,
+        encoding,
+        errors,
+        newline,
+        closefd,
+        opener,
+        source_ref,
+    ):
+        ChildrenHavingFileModeOptionalBufferingOptionalEncodingOptionalErrorsOptionalNewlineOptionalClosefdOptionalOpenerOptionalMixin.__init__(
+            self,
+            file=file,
+            mode=mode,
+            buffering=buffering,
+            encoding=encoding,
+            errors=errors,
+            newline=newline,
+            closefd=closefd,
+            opener=opener,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+        self.attempted = False
+
+    def computeExpression(self, trace_collection):
+        if self.attempted or not builtins_open_since_3_spec.isCompileTimeComputable(
+            (
+                self.subnode_file,
+                self.subnode_mode,
+                self.subnode_buffering,
+                self.subnode_encoding,
+                self.subnode_errors,
+                self.subnode_newline,
+                self.subnode_closefd,
+                self.subnode_opener,
+            )
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
 class ExpressionCtypesCdllRef(ExpressionImportModuleNameHardExistsSpecificBase):
     """Function reference ctypes.CDLL"""
 
@@ -175,7 +289,6 @@ class ExpressionCtypesCdllSince38CallBase(
     def __init__(
         self, name, mode, handle, use_errno, use_lasterror, winmode, source_ref
     ):
-
         ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalWinmodeOptionalMixin.__init__(
             self,
             name=name,
@@ -188,8 +301,7 @@ class ExpressionCtypesCdllSince38CallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not ctypes_cdll_since_38_spec.isCompileTimeComputable(
@@ -244,7 +356,6 @@ class ExpressionCtypesCdllBefore38CallBase(
     spec = ctypes_cdll_before_38_spec
 
     def __init__(self, name, mode, handle, use_errno, use_lasterror, source_ref):
-
         ChildrenHavingNameModeOptionalHandleOptionalUseErrnoOptionalUseLasterrorOptionalMixin.__init__(
             self,
             name=name,
@@ -256,8 +367,7 @@ class ExpressionCtypesCdllBefore38CallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not ctypes_cdll_before_38_spec.isCompileTimeComputable(
@@ -345,7 +455,6 @@ class ExpressionImportlibMetadataBackportDistributionCallBase(
     spec = importlib_metadata_backport_distribution_spec
 
     def __init__(self, distribution_name, source_ref):
-
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -439,7 +548,6 @@ class ExpressionImportlibMetadataBackportEntryPointsCallBase(
     spec = importlib_metadata_backport_entry_points_spec
 
     def __init__(self, params, source_ref):
-
         ChildHavingParamsTupleMixin.__init__(
             self,
             params=params,
@@ -533,7 +641,6 @@ class ExpressionImportlibMetadataBackportMetadataCallBase(
     spec = importlib_metadata_backport_metadata_spec
 
     def __init__(self, distribution_name, source_ref):
-
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -627,7 +734,6 @@ class ExpressionImportlibMetadataBackportVersionCallBase(
     spec = importlib_metadata_backport_version_spec
 
     def __init__(self, distribution_name, source_ref):
-
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -721,7 +827,6 @@ class ExpressionImportlibMetadataDistributionCallBase(
     spec = importlib_metadata_distribution_spec
 
     def __init__(self, distribution_name, source_ref):
-
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -729,8 +834,7 @@ class ExpressionImportlibMetadataDistributionCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if (
@@ -835,7 +939,6 @@ class ExpressionImportlibMetadataEntryPointsSince310CallBase(
     spec = importlib_metadata_entry_points_since_310_spec
 
     def __init__(self, params, source_ref):
-
         ChildHavingParamsTupleMixin.__init__(
             self,
             params=params,
@@ -843,8 +946,7 @@ class ExpressionImportlibMetadataEntryPointsSince310CallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if (
@@ -886,11 +988,9 @@ class ExpressionImportlibMetadataEntryPointsBefore310CallBase(
     spec = importlib_metadata_entry_points_before_310_spec
 
     def __init__(self, source_ref):
-
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def finalize(self):
         del self.parent
@@ -973,7 +1073,6 @@ class ExpressionImportlibMetadataMetadataCallBase(
     spec = importlib_metadata_metadata_spec
 
     def __init__(self, distribution_name, source_ref):
-
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -981,8 +1080,7 @@ class ExpressionImportlibMetadataMetadataCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if (
@@ -1067,7 +1165,6 @@ class ExpressionImportlibMetadataVersionCallBase(
     spec = importlib_metadata_version_spec
 
     def __init__(self, distribution_name, source_ref):
-
         ChildHavingDistributionNameMixin.__init__(
             self,
             distribution_name=distribution_name,
@@ -1075,14 +1172,314 @@ class ExpressionImportlibMetadataVersionCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
+        self.attempted = False
+
+    def computeExpression(self, trace_collection):
+        if (
+            self.attempted
+            or not importlib_metadata_version_spec.isCompileTimeComputable(
+                (self.subnode_distribution_name,)
+            )
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
+class ExpressionImportlibResourcesBackportFilesRef(
+    ExpressionImportModuleNameHardExistsSpecificBase
+):
+    """Function reference importlib_resources.files"""
+
+    kind = "EXPRESSION_IMPORTLIB_RESOURCES_BACKPORT_FILES_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name="importlib_resources",
+            import_name="files",
+            module_guaranteed=not shallMakeModule(),
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .PackageResourceNodes import (
+            ExpressionImportlibResourcesBackportFilesCall,
+        )
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=ExpressionImportlibResourcesBackportFilesCall,
+            builtin_spec=importlib_resources_backport_files_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'importlib_resources.files' recognized.",
+        )
+
+
+hard_import_node_classes[
+    ExpressionImportlibResourcesBackportFilesRef
+] = importlib_resources_backport_files_spec
+
+
+class ExpressionImportlibResourcesBackportFilesCallBase(
+    ChildHavingPackageMixin, ExpressionBase
+):
+    """Base class for ImportlibResourcesBackportFilesCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = ("package",)
+
+    __slots__ = ("attempted",)
+
+    spec = importlib_resources_backport_files_spec
+
+    def __init__(self, package, source_ref):
+        ChildHavingPackageMixin.__init__(
+            self,
+            package=package,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
         # In module mode, we expect a changing environment, cannot optimize this
         self.attempted = shallMakeModule()
 
     def computeExpression(self, trace_collection):
         if (
             self.attempted
-            or not importlib_metadata_version_spec.isCompileTimeComputable(
-                (self.subnode_distribution_name,)
+            or not importlib_resources_backport_files_spec.isCompileTimeComputable(
+                (self.subnode_package,)
+            )
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
+class ExpressionImportlibResourcesBackportReadBinaryRef(
+    ExpressionImportModuleNameHardExistsSpecificBase
+):
+    """Function reference importlib_resources.read_binary"""
+
+    kind = "EXPRESSION_IMPORTLIB_RESOURCES_BACKPORT_READ_BINARY_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name="importlib_resources",
+            import_name="read_binary",
+            module_guaranteed=not shallMakeModule(),
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .PackageResourceNodes import (
+            ExpressionImportlibResourcesBackportReadBinaryCall,
+        )
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=ExpressionImportlibResourcesBackportReadBinaryCall,
+            builtin_spec=importlib_resources_backport_read_binary_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'importlib_resources.read_binary' recognized.",
+        )
+
+
+hard_import_node_classes[
+    ExpressionImportlibResourcesBackportReadBinaryRef
+] = importlib_resources_backport_read_binary_spec
+
+
+class ExpressionImportlibResourcesBackportReadBinaryCallBase(
+    ExpressionBytesShapeExactMixin, ChildrenHavingPackageResourceMixin, ExpressionBase
+):
+    """Base class for ImportlibResourcesBackportReadBinaryCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = (
+        "package",
+        "resource",
+    )
+
+    __slots__ = ("attempted",)
+
+    spec = importlib_resources_backport_read_binary_spec
+
+    def __init__(self, package, resource, source_ref):
+        ChildrenHavingPackageResourceMixin.__init__(
+            self,
+            package=package,
+            resource=resource,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+        # In module mode, we expect a changing environment, cannot optimize this
+        self.attempted = shallMakeModule()
+
+    def computeExpression(self, trace_collection):
+        if (
+            self.attempted
+            or not importlib_resources_backport_read_binary_spec.isCompileTimeComputable(
+                (
+                    self.subnode_package,
+                    self.subnode_resource,
+                )
+            )
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
+class ExpressionImportlibResourcesBackportReadTextRef(
+    ExpressionImportModuleNameHardExistsSpecificBase
+):
+    """Function reference importlib_resources.read_text"""
+
+    kind = "EXPRESSION_IMPORTLIB_RESOURCES_BACKPORT_READ_TEXT_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name="importlib_resources",
+            import_name="read_text",
+            module_guaranteed=not shallMakeModule(),
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .PackageResourceNodes import (
+            makeExpressionImportlibResourcesBackportReadTextCall,
+        )
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=makeExpressionImportlibResourcesBackportReadTextCall,
+            builtin_spec=importlib_resources_backport_read_text_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'importlib_resources.read_text' recognized.",
+        )
+
+
+hard_import_node_classes[
+    ExpressionImportlibResourcesBackportReadTextRef
+] = importlib_resources_backport_read_text_spec
+
+
+class ExpressionImportlibResourcesBackportReadTextCallBase(
+    ExpressionStrShapeExactMixin,
+    ChildrenHavingPackageResourceEncodingOptionalErrorsOptionalMixin,
+    ExpressionBase,
+):
+    """Base class for ImportlibResourcesBackportReadTextCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = (
+        "package",
+        "resource",
+        "encoding|optional",
+        "errors|optional",
+    )
+
+    __slots__ = ("attempted",)
+
+    spec = importlib_resources_backport_read_text_spec
+
+    def __init__(self, package, resource, encoding, errors, source_ref):
+        ChildrenHavingPackageResourceEncodingOptionalErrorsOptionalMixin.__init__(
+            self,
+            package=package,
+            resource=resource,
+            encoding=encoding,
+            errors=errors,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+        # In module mode, we expect a changing environment, cannot optimize this
+        self.attempted = shallMakeModule()
+
+    def computeExpression(self, trace_collection):
+        if (
+            self.attempted
+            or not importlib_resources_backport_read_text_spec.isCompileTimeComputable(
+                (
+                    self.subnode_package,
+                    self.subnode_resource,
+                    self.subnode_encoding,
+                    self.subnode_errors,
+                )
             )
         ):
             trace_collection.onExceptionRaiseExit(BaseException)
@@ -1159,7 +1556,6 @@ class ExpressionImportlibResourcesFilesCallBase(
     spec = importlib_resources_files_spec
 
     def __init__(self, package, source_ref):
-
         ChildHavingPackageMixin.__init__(
             self,
             package=package,
@@ -1167,8 +1563,7 @@ class ExpressionImportlibResourcesFilesCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if (
@@ -1256,7 +1651,6 @@ class ExpressionImportlibResourcesReadBinaryCallBase(
     spec = importlib_resources_read_binary_spec
 
     def __init__(self, package, resource, source_ref):
-
         ChildrenHavingPackageResourceMixin.__init__(
             self,
             package=package,
@@ -1265,8 +1659,7 @@ class ExpressionImportlibResourcesReadBinaryCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if (
@@ -1361,7 +1754,6 @@ class ExpressionImportlibResourcesReadTextCallBase(
     spec = importlib_resources_read_text_spec
 
     def __init__(self, package, resource, encoding, errors, source_ref):
-
         ChildrenHavingPackageResourceEncodingOptionalErrorsOptionalMixin.__init__(
             self,
             package=package,
@@ -1372,8 +1764,7 @@ class ExpressionImportlibResourcesReadTextCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if (
@@ -1455,7 +1846,6 @@ class ExpressionOsListdirCallBase(ChildHavingPathOptionalMixin, ExpressionBase):
     spec = os_listdir_spec
 
     def __init__(self, path, source_ref):
-
         ChildHavingPathOptionalMixin.__init__(
             self,
             path=path,
@@ -1463,8 +1853,7 @@ class ExpressionOsListdirCallBase(ChildHavingPathOptionalMixin, ExpressionBase):
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_listdir_spec.isCompileTimeComputable(
@@ -1538,7 +1927,6 @@ class ExpressionOsPathAbspathCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_abspath_spec
 
     def __init__(self, path, source_ref):
-
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -1546,8 +1934,7 @@ class ExpressionOsPathAbspathCallBase(ChildHavingPathMixin, ExpressionBase):
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_path_abspath_spec.isCompileTimeComputable(
@@ -1621,7 +2008,6 @@ class ExpressionOsPathBasenameCallBase(ChildHavingPMixin, ExpressionBase):
     spec = os_path_basename_spec
 
     def __init__(self, p, source_ref):
-
         ChildHavingPMixin.__init__(
             self,
             p=p,
@@ -1629,8 +2015,7 @@ class ExpressionOsPathBasenameCallBase(ChildHavingPMixin, ExpressionBase):
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_path_basename_spec.isCompileTimeComputable(
@@ -1654,6 +2039,87 @@ class ExpressionOsPathBasenameCallBase(ChildHavingPMixin, ExpressionBase):
         return True
 
 
+class ExpressionOsPathDirnameRef(ExpressionImportModuleNameHardExistsSpecificBase):
+    """Function reference os.path.dirname"""
+
+    kind = "EXPRESSION_OS_PATH_DIRNAME_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name=os.path.__name__,
+            import_name="dirname",
+            module_guaranteed=True,
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .OsSysNodes import ExpressionOsPathDirnameCall
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=ExpressionOsPathDirnameCall,
+            builtin_spec=os_path_dirname_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'os.path.dirname' recognized.",
+        )
+
+
+hard_import_node_classes[ExpressionOsPathDirnameRef] = os_path_dirname_spec
+
+
+class ExpressionOsPathDirnameCallBase(ChildHavingPMixin, ExpressionBase):
+    """Base class for OsPathDirnameCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = ("p",)
+
+    __slots__ = ("attempted",)
+
+    spec = os_path_dirname_spec
+
+    def __init__(self, p, source_ref):
+        ChildHavingPMixin.__init__(
+            self,
+            p=p,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+        self.attempted = False
+
+    def computeExpression(self, trace_collection):
+        if self.attempted or not os_path_dirname_spec.isCompileTimeComputable(
+            (self.subnode_p,)
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
 class ExpressionOsPathExistsRef(ExpressionImportModuleNameHardExistsSpecificBase):
     """Function reference os.path.exists"""
 
@@ -1704,7 +2170,6 @@ class ExpressionOsPathExistsCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_exists_spec
 
     def __init__(self, path, source_ref):
-
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -1712,8 +2177,7 @@ class ExpressionOsPathExistsCallBase(ChildHavingPathMixin, ExpressionBase):
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_path_exists_spec.isCompileTimeComputable(
@@ -1789,7 +2253,6 @@ class ExpressionOsPathIsabsCallBase(
     spec = os_path_isabs_spec
 
     def __init__(self, s, source_ref):
-
         ChildHavingSMixin.__init__(
             self,
             s=s,
@@ -1797,8 +2260,7 @@ class ExpressionOsPathIsabsCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_path_isabs_spec.isCompileTimeComputable(
@@ -1872,7 +2334,6 @@ class ExpressionOsPathIsdirCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_isdir_spec
 
     def __init__(self, path, source_ref):
-
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -1880,8 +2341,7 @@ class ExpressionOsPathIsdirCallBase(ChildHavingPathMixin, ExpressionBase):
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_path_isdir_spec.isCompileTimeComputable(
@@ -1955,7 +2415,6 @@ class ExpressionOsPathIsfileCallBase(ChildHavingPathMixin, ExpressionBase):
     spec = os_path_isfile_spec
 
     def __init__(self, path, source_ref):
-
         ChildHavingPathMixin.__init__(
             self,
             path=path,
@@ -1963,8 +2422,7 @@ class ExpressionOsPathIsfileCallBase(ChildHavingPathMixin, ExpressionBase):
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not os_path_isfile_spec.isCompileTimeComputable(
@@ -2036,11 +2494,9 @@ class ExpressionOsUnameCallBase(ExpressionBase):
     spec = os_uname_spec
 
     def __init__(self, source_ref):
-
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def finalize(self):
         del self.parent
@@ -2123,7 +2579,6 @@ class ExpressionPkgResourcesGetDistributionCallBase(
     spec = pkg_resources_get_distribution_spec
 
     def __init__(self, dist, source_ref):
-
         ChildHavingDistMixin.__init__(
             self,
             dist=dist,
@@ -2220,7 +2675,6 @@ class ExpressionPkgResourcesIterEntryPointsCallBase(
     spec = pkg_resources_iter_entry_points_spec
 
     def __init__(self, group, name, source_ref):
-
         ChildrenHavingGroupNameOptionalMixin.__init__(
             self,
             group=group,
@@ -2314,7 +2768,6 @@ class ExpressionPkgResourcesRequireCallBase(
     spec = pkg_resources_require_spec
 
     def __init__(self, requirements, source_ref):
-
         ChildHavingRequirementsTupleMixin.__init__(
             self,
             requirements=requirements,
@@ -2408,7 +2861,6 @@ class ExpressionPkgResourcesResourceStreamCallBase(
     spec = pkg_resources_resource_stream_spec
 
     def __init__(self, package_or_requirement, resource_name, source_ref):
-
         ChildrenHavingPackageOrRequirementResourceNameMixin.__init__(
             self,
             package_or_requirement=package_or_requirement,
@@ -2509,7 +2961,6 @@ class ExpressionPkgResourcesResourceStringCallBase(
     spec = pkg_resources_resource_string_spec
 
     def __init__(self, package_or_requirement, resource_name, source_ref):
-
         ChildrenHavingPackageOrRequirementResourceNameMixin.__init__(
             self,
             package_or_requirement=package_or_requirement,
@@ -2604,7 +3055,6 @@ class ExpressionPkgutilGetDataCallBase(
     spec = pkgutil_get_data_spec
 
     def __init__(self, package, resource, source_ref):
-
         ChildrenHavingPackageResourceMixin.__init__(
             self,
             package=package,
@@ -2613,8 +3063,7 @@ class ExpressionPkgutilGetDataCallBase(
 
         ExpressionBase.__init__(self, source_ref)
 
-        # In module mode, we expect a changing environment, cannot optimize this
-        self.attempted = shallMakeModule()
+        self.attempted = False
 
     def computeExpression(self, trace_collection):
         if self.attempted or not pkgutil_get_data_spec.isCompileTimeComputable(
@@ -2639,3 +3088,101 @@ class ExpressionPkgutilGetDataCallBase(
     @staticmethod
     def mayRaiseExceptionOperation():
         return True
+
+
+class ExpressionSysExitRef(ExpressionImportModuleNameHardExistsSpecificBase):
+    """Function reference sys.exit"""
+
+    kind = "EXPRESSION_SYS_EXIT_REF"
+
+    def __init__(self, source_ref):
+        ExpressionImportModuleNameHardExistsSpecificBase.__init__(
+            self,
+            module_name="sys",
+            import_name="exit",
+            module_guaranteed=True,
+            source_ref=source_ref,
+        )
+
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
+        # Anything may happen on call trace before this. On next pass, if
+        # replaced, we might be better but not now.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        from .OsSysNodes import makeExpressionSysExitCall
+
+        result = extractBuiltinArgs(
+            node=call_node,
+            builtin_class=makeExpressionSysExitCall,
+            builtin_spec=sys_exit_spec,
+        )
+
+        return (
+            result,
+            "new_expression",
+            "Call to 'sys.exit' recognized.",
+        )
+
+
+hard_import_node_classes[ExpressionSysExitRef] = sys_exit_spec
+
+
+class ExpressionSysExitCallBase(ChildHavingExitCodeOptionalMixin, ExpressionBase):
+    """Base class for SysExitCall
+
+    Generated boiler plate code.
+    """
+
+    named_children = ("exit_code|optional",)
+
+    __slots__ = ("attempted",)
+
+    spec = sys_exit_spec
+
+    def __init__(self, exit_code, source_ref):
+        ChildHavingExitCodeOptionalMixin.__init__(
+            self,
+            exit_code=exit_code,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+        # In module mode, we expect a changing environment, cannot optimize this
+        self.attempted = shallMakeModule()
+
+    def computeExpression(self, trace_collection):
+        if self.attempted or not sys_exit_spec.isCompileTimeComputable(
+            (self.subnode_exit_code,)
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        try:
+            return self.replaceWithCompileTimeValue(trace_collection)
+        finally:
+            self.attempted = True
+
+    @abstractmethod
+    def replaceWithCompileTimeValue(self, trace_collection):
+        pass
+
+    @staticmethod
+    def mayRaiseExceptionOperation():
+        return True
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
