@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """
 
 Plugins: Welcome to Nuitka! This is your shortest way to become part of it.
@@ -42,6 +28,7 @@ from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.freezer.IncludedDataFiles import IncludedDataFile
 from nuitka.freezer.IncludedEntryPoints import IncludedEntryPoint
 from nuitka.ModuleRegistry import addUsedModule
+from nuitka.PythonVersions import python_version
 from nuitka.Tracing import plugins_logger, printLine
 from nuitka.utils.FileOperations import (
     getDllBasename,
@@ -118,10 +105,8 @@ def _addActivePlugin(plugin_class, args, force=False):
     else:
         plugin_args = {}
 
-    try:
+    with withPluginProblemReporting(plugin_class, "Plugin initialization failed", ()):
         plugin_instance = plugin_class(**plugin_args)
-    except TypeError as e:
-        plugin_class.sysexit("Problem initializing plugin: %s" % e)
 
     assert isinstance(plugin_instance, NuitkaPluginBase), plugin_instance
 
@@ -295,7 +280,10 @@ def _loadPluginClassesFromPackage(scan_package):
         if item.ispkg:  # spell-checker: ignore ispkg
             continue
 
-        module_loader = item.module_finder.find_module(item.name)
+        if python_version < 0x3C0:
+            module_loader = item.module_finder.find_module(item.name)
+        else:
+            module_loader = item.module_finder.find_spec(item.name).loader
 
         # Ignore bytecode only left overs.
         try:
@@ -460,9 +448,9 @@ class Plugins(object):
         return result
 
     @staticmethod
-    def _reportImplicitImports(plugin, module, implicit_imports, signal_change):
-        from nuitka.importing import Recursion
+    def _reportImplicitImports(plugin, module, implicit_imports):
         from nuitka.importing.Importing import getModuleNameAndKindFromFilename
+        from nuitka.importing.Recursion import decideRecursion, recurseTo
 
         for full_name, module_filename in implicit_imports:
             # TODO: The module_kind should be forwarded from previous in the class using locateModule code.
@@ -471,7 +459,7 @@ class Plugins(object):
             )
 
             # This will get back to all other plugins allowing them to inhibit it though.
-            decision, reason = Recursion.decideRecursion(
+            decision, decision_reason = decideRecursion(
                 using_module_name=module.getFullName(),
                 module_filename=module_filename,
                 module_name=full_name,
@@ -479,21 +467,20 @@ class Plugins(object):
             )
 
             if decision:
-                imported_module = Recursion.recurseTo(
-                    signal_change=signal_change,
+                imported_module = recurseTo(
                     module_name=full_name,
                     module_filename=module_filename,
                     module_kind=module_kind,
-                    using_module_name=module.module_name,
                     source_ref=module.getSourceReference(),
-                    reason=reason,
+                    reason="implicit import",
+                    using_module_name=module.module_name,
                 )
 
                 addUsedModule(
                     module=imported_module,
                     using_module=module,
                     usage_tag="plugin:" + plugin.plugin_name,
-                    reason=reason,
+                    reason=decision_reason,
                     source_ref=module.source_ref,
                 )
 
@@ -523,7 +510,7 @@ class Plugins(object):
         return cls.extra_scan_paths_cache[key]
 
     @classmethod
-    def considerImplicitImports(cls, module, signal_change):
+    def considerImplicitImports(cls, module):
         for plugin in getActivePlugins():
             key = (module.getFullName(), plugin)
 
@@ -537,7 +524,6 @@ class Plugins(object):
                 plugin=plugin,
                 module=module,
                 implicit_imports=cls.implicit_imports_cache[key],
-                signal_change=signal_change,
             )
 
         # Pre and post load code may have been created, if so indicate it's used.
@@ -796,16 +782,17 @@ class Plugins(object):
             putTextFileContents(filename=source_path, contents=code)
 
         try:
-            trigger_module, _added = buildModule(
+            trigger_module = buildModule(
                 module_filename=os.path.join(
                     os.path.dirname(module.getCompileTimeFilename()),
                     module_name.asPath() + ".py",
                 ),
                 module_name=module_name,
+                reason="trigger",
                 source_code=code,
                 is_top=False,
                 is_main=False,
-                is_extension=False,
+                module_kind="py",
                 is_fake=module_name,
                 hide_syntax_error=False,
             )
@@ -843,7 +830,10 @@ class Plugins(object):
                         flags = ()
                     else:
                         code, reason, flags = desc
-                        if type(flags) is str:
+
+                        if flags is None:
+                            flags = ()
+                        elif type(flags) is str:
                             flags = (flags,)
 
                     yield plugin, code, reason, flags
@@ -881,21 +871,14 @@ class Plugins(object):
                 _untangleFakeDesc(description=plugin.createFakeModuleDependency(module))
             )
 
-        def combineLoadCodes(module_load_descriptions, mode):
+        def combineLoadCodes(module_load_descriptions):
             future_imports_code = []
             normal_code_code = []
             total_flags = OrderedSet()
             reasons = []
 
-            for plugin, code, reason, flags in module_load_descriptions:
+            for _plugin, code, reason, flags in module_load_descriptions:
                 if code:
-                    plugin.info(
-                        "Injecting %s-module load code for module '%s':"
-                        % (mode, full_name)
-                    )
-                    for line in reason.split("\n"):
-                        plugin.info("    " + line)
-
                     for line in code.splitlines():
                         line = line + "\n"
 
@@ -913,7 +896,7 @@ class Plugins(object):
 
         if pre_module_load_descriptions:
             total_code, reasons, total_flags = combineLoadCodes(
-                module_load_descriptions=pre_module_load_descriptions, mode="pre"
+                module_load_descriptions=pre_module_load_descriptions
             )
 
             if total_code:
@@ -929,7 +912,7 @@ class Plugins(object):
 
         if post_module_load_descriptions:
             total_code, reasons, total_flags = combineLoadCodes(
-                module_load_descriptions=post_module_load_descriptions, mode="post"
+                module_load_descriptions=post_module_load_descriptions
             )
 
             if total_code:
@@ -955,13 +938,14 @@ class Plugins(object):
                 fake_filename,
                 reason,
             ) in fake_module_descriptions:
-                fake_module, _added = buildModule(
+                fake_module = buildModule(
                     module_filename=fake_filename,
                     module_name=fake_module_name,
+                    reason="fake",
                     source_code=source_code,
                     is_top=False,
                     is_main=False,
-                    is_extension=False,
+                    module_kind="py",
                     is_fake=fake_module_name,
                     hide_syntax_error=False,
                 )
@@ -972,7 +956,7 @@ class Plugins(object):
                 fake_modules[full_name].append((fake_module, plugin, reason))
 
     @staticmethod
-    def onModuleSourceCode(module_name, source_code):
+    def onModuleSourceCode(module_name, source_filename, source_code):
         assert type(module_name) is ModuleName
         assert type(source_code) is str
 
@@ -980,7 +964,11 @@ class Plugins(object):
 
         for plugin in getActivePlugins():
             with withPluginModuleNameProblemReporting(plugin, module_name):
-                new_source_code = plugin.onModuleSourceCode(module_name, source_code)
+                new_source_code = plugin.onModuleSourceCode(
+                    module_name=module_name,
+                    source_filename=source_filename,
+                    source_code=source_code,
+                )
                 if new_source_code is not None and new_source_code != source_code:
                     source_code = new_source_code
                     contributing_plugins.add(plugin)
@@ -989,19 +977,6 @@ class Plugins(object):
 
         return source_code, contributing_plugins
 
-    @staticmethod
-    def onFrozenModuleSourceCode(module_name, is_package, source_code):
-        assert type(module_name) is ModuleName
-        assert type(source_code) is str
-
-        for plugin in getActivePlugins():
-            source_code = plugin.onFrozenModuleSourceCode(
-                module_name, is_package, source_code
-            )
-            assert type(source_code) is str
-
-        return source_code
-
     @staticmethod
     def onFrozenModuleBytecode(module_name, is_package, bytecode):
         assert type(module_name) is ModuleName
@@ -1038,17 +1013,60 @@ class Plugins(object):
                 if result[0] != must_recurse[0]:
                     plugin.sysexit(
                         "Error, decision %s does not match other plugin '%s' decision."
-                        % (must_recurse[0], ".".join(deciding_plugins))
+                        % (
+                            must_recurse[0],
+                            ".".join(
+                                deciding_plugin.plugin_name
+                                for deciding_plugin in deciding_plugins
+                            ),
+                        )
                     )
 
-            deciding_plugins.append(plugin.plugin_name)
+            deciding_plugins.append(plugin)
             result = must_recurse
 
-        return result
+        return result, deciding_plugins
+
+    module_usage_looked_ahead_cache = set()
+
+    @classmethod
+    def onModuleUsageLookAhead(cls, module_name, module_filename, module_kind):
+        if module_name in cls.module_usage_looked_ahead_cache:
+            return
+
+        cls.module_usage_looked_ahead_cache.add(module_name)
+
+        # Lazy load the source code if a plugin wants it, the pre_load caches
+        # the result for later usage.
+        def getModuleSourceCode():
+            if module_kind != "py":
+                return None
+
+            from nuitka.tree.SourceHandling import readSourceCodeFromFilename
+
+            if os.path.isdir(module_filename):
+                source_filename = os.path.join(module_filename, "__init__.py")
+            else:
+                source_filename = module_filename
+
+            if not os.path.isfile(source_filename):
+                return None
+
+            return readSourceCodeFromFilename(
+                module_name=module_name, source_filename=source_filename, pre_load=True
+            )
+
+        for plugin in getActivePlugins():
+            plugin.onModuleUsageLookAhead(
+                module_name=module_name,
+                module_filename=module_filename,
+                module_kind=module_kind,
+                get_module_source=getModuleSourceCode,
+            )
 
     @staticmethod
     def onModuleRecursion(
-        module_name, module_filename, module_kind, using_module_name, source_ref
+        module_name, module_filename, module_kind, using_module_name, source_ref, reason
     ):
         for plugin in getActivePlugins():
             plugin.onModuleRecursion(
@@ -1057,6 +1075,7 @@ class Plugins(object):
                 module_kind=module_kind,
                 using_module_name=using_module_name,
                 source_ref=source_ref,
+                reason=reason,
             )
 
     @staticmethod
@@ -1223,7 +1242,7 @@ class Plugins(object):
                 # We order per plugin, but from the plugins, lets just take a dict
                 # and achieve determinism by ordering the files by name.
                 for key, value in sorted(value.items()):
-                    if (for_onefile and not "onefile_" in key) or (
+                    if (for_onefile and "onefile_" not in key) or (
                         not for_onefile and "onefile_" in key
                     ):
                         continue
@@ -1309,9 +1328,9 @@ class Plugins(object):
 
     @classmethod
     def encodeDataComposerName(cls, name):
+        # Encoding needs to match generated source code output.
         if str is not bytes:
-            # Encoding needs to match generated source code output.
-            name = name.encode("latin1")
+            name = name.encode("utf8")
 
         for plugin in getActivePlugins():
             r = plugin.encodeDataComposerName(name)
@@ -1342,42 +1361,46 @@ class Plugins(object):
                 yield value
 
     @classmethod
-    def decideAllowOutsideDependencies(cls, module_name):
-        if not Options.isExperimental("no-outside-dependencies"):
-            return None
+    def getExtraConstantDefaultPopulation(cls):
+        for plugin in getActivePlugins():
+            for value in plugin.getExtraConstantDefaultPopulation():
+                yield value
 
+    @classmethod
+    def decideAllowOutsideDependencies(cls, module_name):
         result = None
         plugin_name = None
 
-        if module_name is not None:
-            for plugin in getActivePlugins():
-                value = plugin.decideAllowOutsideDependencies(module_name)
+        assert module_name is not None
 
-                if value is True:
-                    if result is False:
-                        plugin.sysexit(
-                            "Error, conflicting allow/disallow outside dependencies of plug-in '%s'."
-                            % plugin_name
-                        )
+        for plugin in getActivePlugins():
+            value = plugin.decideAllowOutsideDependencies(module_name)
 
-                    result = True
-                    plugin_name = plugin.plugin_name
-
-                elif value is False:
-                    if result is False:
-                        plugin.sysexit(
-                            "Error, conflicting allow/disallow outside dependencies of plug-in '%s'."
-                            % plugin_name
-                        )
-
-                    result = False
-                    plugin_name = plugin.plugin_name
-                elif value is not None:
+            if value is True:
+                if result is False:
                     plugin.sysexit(
-                        "Error, can only return True, False, None from 'decideAllowOutsideDependencies' not %r"
-                        % value
+                        "Error, conflicting allow/disallow outside dependencies of plug-in '%s'."
+                        % plugin_name
                     )
 
+                result = True
+                plugin_name = plugin.plugin_name
+
+            elif value is False:
+                if result is False:
+                    plugin.sysexit(
+                        "Error, conflicting allow/disallow outside dependencies of plug-in '%s'."
+                        % plugin_name
+                    )
+
+                result = False
+                plugin_name = plugin.plugin_name
+            elif value is not None:
+                plugin.sysexit(
+                    "Error, can only return True, False, None from 'decideAllowOutsideDependencies' not %r"
+                    % value
+                )
+
         return result
 
     @classmethod
@@ -1741,3 +1764,19 @@ def replaceTriggerModule(old, new):
 def isTriggerModule(module):
     """Decide of a module is a trigger module."""
     return module in pre_modules.values() or module in post_modules.values()
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
