@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ This module maintains the parameter specification classes.
 
 These are used for function, lambdas, generators. They are also a factory
@@ -512,7 +498,7 @@ def matchCall(
 
     # Fill in any missing values with the None to indicate "default".
     if num_defaults > 0:
-        for arg in (kw_only_args + args)[-num_defaults:]:
+        for arg in (args + kw_only_args)[-num_defaults:]:
             if not isAssigned(arg):
                 assign(arg, None)
 
@@ -591,11 +577,26 @@ def matchCall(
                     unassigned,
                     "s" if unassigned > 1 else "",
                     " and ".join(
-                        "'%s'"
-                        % [arg.getName() for arg in kw_only_args if not isAssigned(arg)]
+                        "'%s'" % [arg for arg in kw_only_args if not isAssigned(arg)]
                     ),
                 )
             )
         )
 
     return result
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
