@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Reformulation of function statements.
 
 Consult the Developer Manual for information. TODO: Add ability to sync
@@ -214,6 +200,7 @@ def _buildBytecodeOrSourceFunction(provider, node, compilation_mode, source_ref)
                         source_code=source,
                         globals_arg=globals_ref,
                         locals_arg=locals_ref,
+                        closure=None,
                         source_ref=source_ref,
                     ),
                     source_ref=source_ref,
@@ -251,6 +238,8 @@ def buildFunctionNode(provider, node, source_ref):
     )
 
     if compilation_mode != "compiled":
+        node.name = mangleName(node.name, provider)
+
         return _buildBytecodeOrSourceFunction(
             provider=provider,
             node=node,
@@ -587,8 +576,8 @@ def buildParameterAnnotations(provider, node, source_ref):
     # Too many branches, because there is too many cases, pylint: disable=too-many-branches
 
     # The ast uses funny names a bunch.
-    # spellchecker: ignore varnames,elts,posonlyargs,kwonlyargs,varargannotation,vararg
-    # spellchecker: ignore kwargannotation
+    # spell-checker: ignore elts,posonlyargs,kwonlyargs,varargannotation,vararg
+    # spell-checker: ignore kwargannotation
 
     # Build annotations. We are hiding here, that it is a Python3 only feature.
     if python_version < 0x300 or hasPythonFlagNoAnnotations():
@@ -680,7 +669,9 @@ def _wrapFunctionWithSpecialNestedArgs(
 
         sub_special_index = 0
 
-        iter_var = outer_body.allocateTempVariable(None, "arg_iter_%d" % len(iter_vars))
+        iter_var = outer_body.allocateTempVariable(
+            temp_scope=None, name="arg_iter_%d" % len(iter_vars), temp_type="object"
+        )
         iter_vars.append(iter_var)
 
         statements.append(
@@ -924,3 +915,19 @@ def addFunctionVariableReleases(function):
         )
 
         function.setChildBody(makeStatementsSequenceFromStatement(statement=body))
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
