@@ -1,20 +1,6 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Python tests originally created or extracted from other peoples work. The
-#     parts were too small to be protected.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Test that should cover sporadic usages of built-ins that we implemented.
 
 """
@@ -741,7 +727,7 @@ print(zip([1, 2, 3], "String"))
 try:
     zip(1, "String")
 except TypeError as e:
-    print("Occured", repr(e))
+    print("Occurred", repr(e))
 print(zip())
 x = [(u, v) for (u, v) in zip(range(8), reversed(range(8)))]
 print(x)
@@ -752,3 +738,18 @@ for v in zip([1, 2, 3], "String"):
 # that propagated as call argument.
 func = "{foo}".format
 print(func(foo="Foo"))
+
+#     Python tests originally created or extracted from other peoples work. The
+#     parts were too small to be protected.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
