@@ -1,23 +1,6 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Python test originally created or extracted from other peoples work. The
-#     parts from me are licensed as below. It is at least Free Software where
-#     it's copied from other people. In these cases, that will normally be
-#     indicated.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#         http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
 
 """ Runner for plugins tests of Nuitka.
 
@@ -41,27 +24,18 @@ sys.path.insert(
 from nuitka.tools.testing.Common import (
     compareWithCPython,
     createSearchMode,
+    getMainProgramFilename,
     my_print,
+    reportSkip,
     setup,
     withPythonPathChange,
 )
-
-
-def getMainProgramFilename(filename):
-    for filename_main in os.listdir(filename):
-        if filename_main.endswith(("Main.py", "Main")):
-            return filename_main
-
-    sys.exit(
-        """\
-Error, no file ends with 'Main.py' or 'Main' in %s, incomplete test case."""
-        % (filename)
-    )
+from nuitka.Version import getCommercialVersion
 
 
 def main():
     # Complex stuff, even more should become common code though.
-    setup(needs_io_encoding=True)
+    setup(suite="plugins", needs_io_encoding=True)
 
     search_mode = createSearchMode()
 
@@ -81,17 +55,17 @@ def main():
         extra_flags = ["expect_success"]
 
         # We annotate some tests, use that to lower warnings.
-        extra_flags.append("plugin_enable:pylint-warnings")
         extra_flags.append("remove_output")
-        extra_flags.append("--follow-imports")
 
-        plugin_files = [p for p in os.listdir(filename) if p.endswith("-plugin.py")]
+        user_plugin_files = [
+            p for p in os.listdir(filename) if p.endswith("-plugin.py")
+        ]
 
-        assert plugin_files
-        extra_flags.extend(
-            "user_plugin:" + os.path.abspath(os.path.join(filename, p))
-            for p in plugin_files
-        )
+        if user_plugin_files:
+            extra_flags.extend(
+                "user_plugin:" + os.path.abspath(os.path.join(filename, p))
+                for p in user_plugin_files
+            )
 
         if filename == "parameters":
             os.environ["NUITKA_EXTRA_OPTIONS"] = extra_options + " --trace-my-plugin"
@@ -103,6 +77,13 @@ def main():
         if active:
             my_print("Consider output of recursively compiled program:", filename)
 
+            if filename in ("code_signing",):
+                if getCommercialVersion() is None:
+                    reportSkip(
+                        "Plugin only available in Nuitka commercial", ".", filename
+                    )
+                    continue
+
             filename_main = getMainProgramFilename(filename)
 
             extra_python_path = [
@@ -125,3 +106,20 @@ def main():
 
 if __name__ == "__main__":
     main()
+
+#     Python test originally created or extracted from other peoples work. The
+#     parts from me are licensed as below. It is at least Free Software where
+#     it's copied from other people. In these cases, that will normally be
+#     indicated.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
